﻿/* ZZJ QUICKEDITOR v1.0 *
  
 Features:
 * Regex search and replace: multiline, JS-compat, and case-insensitive mode
 * Escape sequences supported for replace and generic non-regex search
 * Supports loading, saving, and hotkey support
 * File associations and drag and drop file loading
         
 Limitations/Quirks:
 * Newlines in file contents will be regularlized to \r\n when saved
 * \n escape in search will be automatically changed to \r\n, internally
 * \r escape sequence is recognized but supressed (removed from search)
 * Binary files usually do not load properly and will show an error
 * Undo is single level only
   
 Escape equences supported: \n \t \0 \b
 
 *** HOTKEYS ***    
  ctrl + a : select all
  ctrl cvx : clipboard operations
  ctrl + s : save
  ctrl + z : undo/redo
  ctrl + l : load file
  ctrl + f : find next occurance of currently selected text
  ctrl + r : focus the replace box
  ctrl + n : new file
  F3       : find next occurance of text in search box
  F5       : reload file from disk (prompt to save 1st)
  ctrl + v : paste
  ctrl + x : cut
  ctrl + c : copy

 */

using System;
using System.Windows.Forms;

namespace Editor {
    public partial class EditorForm : Form {

        public EditorForm() : this(null) { }
        public EditorForm(string file) {

            InitializeComponent();

            if (file == null) {
                NewFile();
            } else
                if (!LoadFromFile(file)) {
                    this.Dispose();
                    throw new Exception();
                }

            textBox.HideSelection = false;
            textBox.ShortcutsEnabled = true;
            textBox.AllowDrop = true;

            textBox.DragDrop += new DragEventHandler(DDEvent);
            textBox.DragEnter += new DragEventHandler(DDEnter);
     
            textBox.TextChanged += new EventHandler(tbContents_TextChanged);

            this.Shown += new EventHandler(EditorForm_Shown);
            this.KeyDown += new KeyEventHandler(EditorForm_KeyDown);
            this.KeyPreview = true;
        }

        void EditorForm_Shown(object sender, EventArgs e) {
            textBox.Focus();
        }

        private void DDEnter(object sender, DragEventArgs e) {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false))
                e.Effect = DragDropEffects.All;

        }

        private void DDEvent(object sender, DragEventArgs e) {
            string fileName = ((string[])e.Data.GetData(DataFormats.FileDrop))[0];

            LoadFromFile(fileName);
        }

        protected override void OnClosing(System.ComponentModel.CancelEventArgs e) {
            if (PromptDYWTS())
                e.Cancel = true;

            base.OnClosing(e);
        }

        // shortcut keys handling
        void EditorForm_KeyDown(object sender, KeyEventArgs e) {
            if (e.Modifiers == Keys.Control) {
                e.Handled = true;
                e.SuppressKeyPress = true;

                switch (e.KeyCode) {
                    case Keys.S:
                        SaveFile();
                        break;
                    case Keys.A:
                        foreach (Control c in this.Controls)
                            if (c is TextBox && c.Focused) {
                                ((TextBox)c).SelectAll();
                                break;
                            }
                        break;
                    case Keys.V:
                        foreach (Control c in this.Controls)
                            if (c is TextBox && c.Focused) {
                                ((TextBox)c).Paste(FixNewlines(Clipboard.GetText()));
                                break;
                            }
                        break;
                    case Keys.Z:
                        foreach (Control c in this.Controls)
                            if (c is TextBox && c.Focused && ((TextBox)c).CanUndo) {
                                ((TextBox)c).Undo();
                                break;
                            }
                        break;
                    case Keys.L:
                        LoadFile();
                        break;
                    case Keys.F:
                        //if (textBox.SelectedText != "")
                        tbSearch.Text = textBox.SelectedText;
                        FindNext();
                        break;
                    case Keys.R:
                        tbReplace.Focus();
                        break;
                    case Keys.N:
                        NewFile();
                        break;
                    default:
                        e.Handled = false;
                        e.SuppressKeyPress = false;
                        break;
                }
            } else if (e.Modifiers == Keys.None) {
                e.Handled = true;
                e.SuppressKeyPress = true;

                switch (e.KeyCode) {
                    case Keys.F5:
                        if (FileExists) {
                            LoadFromFile(Filename);
                        } else
                            Status = "File has not been saved!";
                        break;
                    case Keys.F3:
                        FindNext();
                        break;
                    default:
                        e.Handled = false;
                        e.SuppressKeyPress = false;
                        break;
                }
            }
        }

        void tbContents_TextChanged(object sender, EventArgs e) {
            Saved = ContentText.GetHashCode() == OrigTextHash;
        }

        private void button1_Click(object sender, EventArgs e) {
            ReplaceAll(tbSearch.Text,tbReplace.Text);
        }

        private void button3_Click(object sender, EventArgs e) {
            FindNext();
        }

        private void button1_Click_1(object sender, EventArgs e) {
            NewFile();
        }

        private void buttLoad_Click(object sender, EventArgs e) {
            LoadFile();
        }

        private void buttSave_Click(object sender, EventArgs e) {
            SaveFile();
        }

        private void buttSaveas_Click(object sender, EventArgs e) {
            SaveAs();
        }

        private void cbRegex_CheckedChanged(object sender, EventArgs e) {
            panelRgxOpts.Enabled = cbRegex.Checked;
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e) {
            textBox.WordWrap = cbWordWrap.Checked;
            textBox.Focus();
        }


    }
}