﻿// This contains the main workings.

using System;
using System.Windows.Forms;
using System.IO;
using System.Drawing;

namespace Editor {
    public partial class EditorForm : Form {

        private bool _saved = true;     // has it been modified?
        private int NewDocNr = 0;       // number of new docs made
        private string Filename = "";   // current file name, if not a new document it will contain a colon
        private int OrigTextHash = 0;   // hash of the original text, for modified

        private string Status {
            get {
                return labelStatus.Text;
            }
            set {
                labelStatus.Text = value;
            }
        }

        private bool Saved { // has the current doc been saved?
            get {
                return _saved;
            }
            set {
                _saved = value;
                labelEditing.Text = "Editing " + (FileExists ? (Filename.Substring(Filename.LastIndexOf("\\") + 1)) : "<" + Filename.Substring(0, Filename.LastIndexOf(".")) + ">");

                if (value) {
                    labelEditing.ForeColor = Color.Black;
                } else {
                    labelEditing.ForeColor = Color.Red;
                    labelEditing.Text += "*";
                }
            }
        }

        private bool FileExists { // if the file has been saved at some point (the filename is an absolute path)
            get {
                return Filename.Contains(":");
            }
        }

        private string ContentText { // replace the content text without clearing the undo buffer
            get {             
                return textBox.Text;
            }
            set {
                textBox.SelectAll();
                textBox.Paste(FixNewlines(value));
                ScrollToTop();
            }
        }

        private void NewFile() {
            if (PromptDYWTS())
               return;
      
            if (!string.IsNullOrEmpty(textBox.Text)) {
                textBox.Clear();
                NewDocNr++;
            }

            textBox.ClearUndo();

            OrigTextHash = ContentText.GetHashCode();
            this.Filename = "new" + NewDocNr + ".txt";
            Status = "Ready";

            Saved = true;
        }

        private void LoadFile() {

            if (!FileExists) {
                openFileDialog.FileName = "";
            } else
                openFileDialog.FileName = Filename;
            
            if (openFileDialog.ShowDialog() != DialogResult.OK)
                return;
       
            LoadFromFile(openFileDialog.FileName);
        }

        private void SaveFile() {
            if (!FileExists) {
                SaveAs();
                return;
            }

            SaveToFile(Filename);
        }

        private bool LoadFromFile(string fileName) {
            if (PromptDYWTS())
                return false;

            try {
                Status = "Opening...";

                string oldContents = ContentText;
                string fileContents = FixNewlines(File.ReadAllText(fileName)).Replace("\0", "");

                textBox.Text = fileContents;
                
                if (textBox.Text.Length != fileContents.Length) {
                    textBox.Text = oldContents;
                    throw new Exception("tried to load a binary file! (expected " + fileContents.Length + ", got " + ContentText.Length + ")");
                }
                
            } catch (Exception ex) {
                Status = "Failed to open file";
                MessageBox.Show(this, "Failed to open file: " + ex.Message, "Load", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            } finally {
                ScrollToTop();
                textBox.ClearUndo();
            }
            
            OrigTextHash = ContentText.GetHashCode();
            this.Filename = fileName;
            Status = "Loaded " + fileName;
            
            Saved = true;
            textBox.Focus();
           
            return true;
        }

        private void SaveToFile(string fileName) {
            try {
                Status = "Saving file...";
                File.WriteAllText(fileName, ContentText);
            } catch (Exception ex) {
                Status = "Failed to save file";
                MessageBox.Show(this, "Failed to save file: " + ex.Message, "Save", MessageBoxButtons.OK, MessageBoxIcon.Error);
                SaveAs();
                return;
            }

            this.Filename = fileName;
            OrigTextHash = ContentText.GetHashCode();
            Status = "Saved to " + fileName;
            Saved = true;
        }

        private void SaveAs() {
            saveFileDialog.FileName = Filename;
           
            if (saveFileDialog.ShowDialog() != DialogResult.OK)
                return;

            SaveToFile(saveFileDialog.FileName);
        }
       
    }
}