﻿// This contains the search mechanics

using System;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace Editor {
    public partial class EditorForm : Form {

        private RegexOptions RgxOptions {
            get {
                RegexOptions ret = RegexOptions.None;

                if (cbRgxMultiline.Checked)     ret |= RegexOptions.Multiline | RegexOptions.Singleline;
                if (cbRgxIgnoreCase.Checked)    ret |= RegexOptions.IgnoreCase;
                if (cbRgxECMA.Checked)          ret |= RegexOptions.ECMAScript;

                return ret;
            }
        }

        private void ReplaceAll(string search, string replace) {

            if (string.IsNullOrEmpty(ContentText)) {
                Status = "Enter some text, first...";
                return;
            }

            if (tbSearch.Text == "") {
                Status = "Enter a search string, first...";
                return;
            }

            if (search == replace) {
                Status = "This operation is pointless.";
                return;
            }

            Status = "Replacing...";
  
            search = search.Replace(@"\r", "").Replace(@"\n",@"\r\n");
            
            string rplc = ParseEscape(replace); // escape the replacement text
            string text = ContentText;          // not a cheap operation to perform, so only do it once.

            try {
                if (cbRegex.Checked) {

                    Match match = Regex.Match(ContentText, search, RgxOptions);

                    if (!match.Success) {
                        SrchNotFound();
                        return;
                    }
                    
                    System.Text.StringBuilder sb = new System.Text.StringBuilder();
                    int matchCount = 0;

                    // 17.344 seconds for 1000 repetitions of replacing \n with </p>\n<p> in 560211 chars (4965 occurances)
                    // my first try... workable, but inefficent

                    /*for (int i = 0; i < text.Length; i++) {
                      // a match with success set to false is returned when no more matches were found
                      if (!match.Success || i < match.Index) { 
                          sb.Append(text[i]);
                      } else {
                          i += match.Length - 1;
                          sb.Append(rplc); // add replacement text
                          match = match.NextMatch();
                          matchCount++;
                      }
                    }*/

                    // 8.046 seconds for 1000 repetitions of replacing of \n with </p>\n<p> in 560211 chars (4965 occurances)
                    // i didn't use this at first as i thought it was failing to replace the searched string with an empty one
                    // turns out i had accidentally put a check for a null/empty string above, so, yeah, i fail. 
                    // though this will not tell you how many occurances were replaced

                    //sb.Append(Regex.Replace(text, search, rplc));

                    // 7.968 seconds for 1000 repetitions of replacing of \n with </p>\n<p> in 560211 chars (4965 occurances)
                    // my second try.
                    // actually a bit faster, and i get a count of matches... neat! 

                    int ind = 0;

                    while (match.Success) {
                        sb.Append(text.Substring(ind, match.Index - ind));
                        sb.Append(rplc);
                        ind = match.Index + match.Length;
                        match = match.NextMatch();
                        matchCount++;
                    }

                    sb.Append(text.Substring(ind)); // append the last bit

                    ContentText = sb.ToString();

                    Status = "Replaced " + matchCount + " occurance" + (matchCount != 1 ? "s" : "") + ".";
                } else {
                    string srch = ParseEscape(search); // escape the search string
                    int count = CountOccurances(text, srch);

                    if (count == 0) {
                        SrchNotFound();
                        return;
                    }

                    ContentText = text.Replace(srch, rplc);

                    Status = "Replaced " + count + " occurance" + (count != 1 ? "s" : "") + ".";                
                }
            } catch (Exception ex) {
                MessageBox.Show(this, "There was an error while searching: " + ex.Message + (cbRegex.Checked ? "\r\n\r\nMake sure your regex syntax is correct." : ""), "Search", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private bool FindNext() {
            if (tbSearch.Text == "" && textBox.SelectedText != "")
                tbSearch.Text = textBox.SelectedText;
            
            return FindNext(false);
        }

        private bool FindNext(bool wrapping) { // if wrapping = true, search the entire thing from the start

            if (textBox.Text == "") {
                Status = "Enter some text, first...";
                return false;
            }

            if (tbSearch.Text == "") {
                Status = "Enter a search string, first...";
                return false;
            }

            Status = "Searching...";
            textBox.Focus();

            string search = tbSearch.Text.Replace(@"\r", "").Replace(@"\n", @"\r\n");
            
            try {
                if (cbRegex.Checked) { // regex
                    Match m = Regex.Match(wrapping ? textBox.Text : textBox.Text.Substring(textBox.SelectionStart + textBox.SelectionLength),
                        search, RgxOptions);

                    if (!m.Success) {
                        if (!wrapping && cbSrchWrap.Checked)
                            return FindNext(true);

                        SrchNotFound();
                        return false;
                    }

                    
                    textBox.SelectionStart = (wrapping ? 0 : (textBox.SelectionStart + textBox.SelectionLength)) + m.Index;
                    textBox.SelectionLength = m.Length;             
                } else { // extended standard
                    string srch = ParseEscape(search);

                    int ind = ContentText.IndexOf(srch, wrapping ? 0 : (textBox.SelectionStart + textBox.SelectionLength));

                    if (ind != -1) {
                        textBox.SelectionStart = ind;
                        textBox.SelectionLength = srch.Length;
                    } else {
                        if (!wrapping && cbSrchWrap.Checked)
                            return FindNext(true);

                        SrchNotFound();
                        return false;
                    }
                }

                textBox.ScrollToCaret();
                        
                if (wrapping) {
                    Status = "Wrapped search!";
                } else
                    Status = "Found @ " + textBox.SelectionStart;
                return true;
            } catch (Exception ex) {
                MessageBox.Show(this, "There was an error while searching: " + ex.Message + (cbRegex.Checked ? "\r\n\r\nMake sure your regex syntax is correct." : ""), "Search", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }
    }
}